/************************************************************************/
/*  TP GPU - GLSL Ensimag 3A                                            */
/*  Sbastien Barbier - EVASION                                         */
/*	janvier 2009														*/
/************************************************************************/

#include "headers.h"
#include "structure.h"
#include "constant.h"
#include "PrintError.h"

void ReadNeighbourFile(FILE *fin, int *n_vertex, int *n_face, Vertex **plist_v, Face **plist_f,
              float *Xmin,
              float *Xmax,
              float *Ymin,
              float *Ymax,
              float *Zmin,
              float *Zmax)
  {
  Vertex *list_v;
  Face *list_f;
  int i;
  double dummy;
  char format[100];

  *Xmin = FLT_MAX; *Ymin = FLT_MAX; *Zmin = FLT_MAX;
  *Xmax = FLT_MIN; *Ymax = FLT_MIN; *Zmax = FLT_MIN;

  fscanf( fin, "%s\n", format);
  if (strcmp( format, "NEIGHBOUR_FILE") != 0) 
    {PrintError( WRONG_FORMAT, "input file: must be in NEIGHBOUR_FILE format");}
    
  fscanf( fin, "%d %d\n", n_vertex, n_face);

  *plist_v = (Vertex*) malloc(sizeof(Vertex) * *n_vertex);
  list_v = *plist_v;
  if (list_v==NULL) {PrintError( MEMORY, "vertices");}

  *plist_f = (Face*) malloc(sizeof(Face) * *n_face);
  list_f = *plist_f;
  if (list_f==NULL) {PrintError( MEMORY, "faces");}

  for (i=0; i<*n_vertex; i++)
    {
    fscanf( fin, "%f %f %f %f %f %f",
                  &list_v[i].V[0], &list_v[i].V[1], &list_v[i].V[2],
                  &dummy, &dummy, &dummy);

    if (*Xmin > list_v[i].V[0]) {*Xmin = list_v[i].V[0];}
    if (*Ymin > list_v[i].V[1]) {*Ymin = list_v[i].V[1];}
    if (*Zmin > list_v[i].V[2]) {*Zmin = list_v[i].V[2];}

    if (*Xmax < list_v[i].V[0]) {*Xmax = list_v[i].V[0];}
    if (*Ymax < list_v[i].V[1]) {*Ymax = list_v[i].V[1];}
    if (*Zmax < list_v[i].V[2]) {*Zmax = list_v[i].V[2];}
    }

  for (i=0; i<*n_face; i++)
    {
    fscanf( fin, "%d %d %d %d %d %d",
                  &list_f[i].V[0], &list_f[i].V[1], &list_f[i].V[2],
                  &list_f[i].F[0], &list_f[i].F[1], &list_f[i].F[2]);
#ifdef _CHANGE_ORIENTATION_
    tmp = list_f[i].V[1];
    list_f[i].V[1] =  list_f[i].V[2];
    list_f[i].V[2] = tmp;
    tmp = list_f[i].F[1];
    list_f[i].F[1] =  list_f[i].F[2];
    list_f[i].F[2] = tmp;
#endif
    }

  fprintf( stderr, "Read NEIGHBOUR_FILE file with %d vertices and %d faces\n", *n_vertex, *n_face);
  }

void WriteNeighbourFile(FILE *fout, int n_vertex, int n_face, Vertex *list_v, Face *list_f)
  {
  int i;

  fprintf( fout, "NEIGHBOUR_FILE\n");
  fprintf( fout, "%d %d\n", n_vertex, n_face);
  for (i=0; i<n_vertex; i++)
    {
    fprintf( fout, "%20.10lf %20.10lf %20.10lf\n", list_v[i].V[0], list_v[i].V[1], list_v[i].V[2]);
    fprintf( fout, "1. 1. 1.\n");
    }
  for (i=0; i<n_face; i++)
    fprintf( fout, "%5d %5d %5d %5d %5d %5d\n",
                    list_f[i].V[0], list_f[i].V[1], list_f[i].V[2],
                    list_f[i].F[0], list_f[i].F[1], list_f[i].F[2]);
  }